/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/////////////////////////
// the original multiset hash buckets
//////////////////////

#include "sequence.h"
#include "msethash.h"
#include <set>

using namespace std;

MSetHash::MSetHash(BitSequence *pat)
  : Hash(pat),
    activeRange((HashIte)NULL,(HashIte)NULL)
{
  fasta=new MHash[hash_size];
  memset(fasta,0,sizeof(MHash)*hash_size);
}

void MSetHash::clear(){
  for(word i=0; i<hash_size; i++) delete fasta[i];
  memset(fasta,0,sizeof(MHash)*hash_size);
}

void MSetHash::add(const HashKey key,const HashVal &val){
  if(!fasta[key])
    fasta[key]=new multiset<HashVal>;
  fasta[key]->insert(val);
}

void MSetHash::lookup(HashKey key,const HashVal &a,LocList &locList){
  for(
      pair<HashIte,HashIte> range(fasta[key]->equal_range(a));
      range.first!=range.second;
      range.first++){
    locList[range.first->seqId()].push_back(*range.first);
  }    
}

void MSetHash::lookup(HashKey key,LocList &locList){
  for(
      pair<HashIte,HashIte> range(fasta[key]->begin(),
				  fasta[key]->end());
      range.first!=range.second;
      range.first++){
    locList[range.first->seqId()].push_back(*range.first);
  }    
}

bool MSetHash::emptyAt(const HashKey key){
  if(!fasta[key] || fasta[key]->empty())
    return false;
  return sizeAt(key)==0;
}

word MSetHash::sizeAt(const HashKey key){
  if(!fasta[key]) return 0;
  else return fasta[key]->size();
}
